@@ -15,7 +15,8 @@ from ..grids.base import GridBase
 from ..tools.docstrings import fill_in_docstring
 from ..tools.misc import Number, number_array
 from ..tools.plotting import PlotReference, plot_on_figure
-from .base import DataFieldBase, FieldBase, OptionalArrayLike
+from ..tools.typing import ArrayLike
+from .base import DataFieldBase, FieldBase
 from .scalar import ScalarField
 
 
@@ -25,7 +26,7 @@ class FieldCollection(FieldBase):
     def __init__(
         self,
         fields: Sequence[DataFieldBase],
-        data: OptionalArrayLike = None,
+        data: ArrayLike = None,
         *,
         copy_fields: bool = False,
         label: Optional[str] = None,
@@ -36,7 +37,7 @@ class FieldCollection(FieldBase):
         Args:
             fields:
                 Sequence of the individual fields
-            data (:class:`numpy.ndarray`):
+            data (:class:`~numpy.ndarray`):
                 Data of the fields. If `None`, the data is instead taken from
                 the individual fields given by `fields`.
             copy_fields (bool):
@@ -223,7 +224,7 @@ class FieldCollection(FieldBase):
         Args:
             attributes (dict):
                 The attributes that describe the current instance
-            data (:class:`numpy.ndarray`, optional):
+            data (:class:`~numpy.ndarray`, optional):
                 Data values at support points of the grid defining all fields
         """
         if "class" in attributes:
@@ -406,12 +407,12 @@ class FieldCollection(FieldBase):
         return results
 
     def copy(
-        self, data: OptionalArrayLike = None, *, label: str = None, dtype=None
+        self, data: ArrayLike = None, *, label: str = None, dtype=None
     ) -> "FieldCollection":
         """return a copy of the data, but not of the grid
 
         Args:
-            data (:class:`numpy.ndarray`, optional):
+            data (:class:`~numpy.ndarray`, optional):
                 Data values at the support points of the grid that define the
                 field. Note that the data is not copied but used directly.
             label (str, optional):
@@ -500,10 +501,20 @@ class FieldCollection(FieldBase):
         return out
 
     @property
-    def integrals(self):
-        """ return the integrals of all fields """
+    def integrals(self) -> List:
+        """integrals of all fields """
         return [field.integral for field in self]
 
+    @property
+    def averages(self) -> List:
+        """ averages of all fields """
+        return [field.average for field in self]
+
+    @property
+    def magnitudes(self) -> np.ndarray:
+        """ :class:`~numpy.ndarray`: scalar magnitudes of all fields """
+        return np.array([field.magnitude for field in self])
+
     def get_line_data(  # type: ignore
         self,
         index: int = 0,
@@ -552,8 +563,10 @@ class FieldCollection(FieldBase):
     @plot_on_figure(update_method="_update_plot")
     def plot(
         self,
-        kind: str = "auto",
-        resize_fig: bool = True,
+        kind: Union[str, Sequence[str]] = "auto",
+        resize_fig=None,
+        figsize="auto",
+        arrangement="horizontal",
         fig=None,
         subplot_args=None,
         **kwargs,
@@ -561,38 +574,85 @@ class FieldCollection(FieldBase):
         r"""visualize all the fields in the collection
 
         Args:
-            kind (str):
-                Determines the visualizations. Supported values are `image`,
-                `line`, `vector`, or `interactive`. Alternatively, `auto`
-                determines the best visualization based on each field itself.
+            kind (str or list of str):
+                Determines the kind of the visualizations. Supported values are `image`,
+                `line`, `vector`, or `interactive`. Alternatively, `auto` determines the
+                best visualization based on each field itself. Instead of a single value
+                for all fields, a list with individual values can be given.
             resize_fig (bool):
                 Whether to resize the figure to adjust to the number of panels
+            figsize (str or tuple of numbers):
+                Determines the figure size. The figure size is unchanged if the string
+                `default` is passed. Conversely, the size is adjusted automatically when
+                `auto` is passed. Finally, a specific figure size can be specified using
+                two values, using :func:`matplotlib.figure.Figure.set_size_inches`.
+            arrangement (str):
+                Determines how the subpanels will be arranged. The default value
+                `horizontal` places all subplots next to each other. The alternative
+                value `vertical` puts them below each other.
             {PLOT_ARGS}
             subplot_args (list):
                 Additional arguments for the specific subplots. Should be a list with a
                 dictionary of arguments for each subplot. Supplying an empty allows to
                 keep the default setting of specific subplots.
             \**kwargs:
-                All additional keyword arguments are forwarded to the actual
-                plotting function of all subplots.
+                All additional keyword arguments are forwarded to the actual plotting
+                function of all subplots.
 
         Returns:
             List of :class:`PlotReference`: Instances that contain information
             to update all the plots with new data later.
         """
-        # disable interactive plotting temporarily
-        # create a plot with all the panels
-        if resize_fig:
-            fig.set_size_inches((4 * len(self), 3), forward=True)
-        (axs,) = fig.subplots(1, len(self), squeeze=False)
+        if resize_fig is not None:
+            # Deprecated this argument on 2021-02-01
+            import warnings
+
+            warnings.warn(
+                "`resize_fig` is a deprecated argument. Use `figsize` directly",
+                DeprecationWarning,
+            )
+
+            if resize_fig is True:
+                figsize = "auto"
+            elif resize_fig is False:
+                figsize = "default"
+            else:
+                raise ValueError
+
+        # set the size of the figure
+        if figsize == "default":
+            pass  # just leave the figure size at its default value
+
+        elif figsize == "auto":
+            # adjust the size of the figure
+            if arrangement == "horizontal":
+                fig.set_size_inches((4 * len(self), 3), forward=True)
+            elif arrangement == "vertical":
+                fig.set_size_inches((4, 3 * len(self)), forward=True)
+
+        else:
+            # assume that an actual tuple is given
+            fig.set_size_inches(figsize, forward=True)
+
+        # create all the subpanels
+        if arrangement == "horizontal":
+            (axs,) = fig.subplots(1, len(self), squeeze=False)
+        elif arrangement == "vertical":
+            axs = fig.subplots(len(self), 1, squeeze=False)
+            axs = [a[0] for a in axs]  # transpose
+        else:
+            raise ValueError(f"Unknown arrangement `{arrangement}`")
 
         if subplot_args is None:
             subplot_args = [{}] * len(self)
 
+        if isinstance(kind, str):
+            kind = [kind] * len(self.fields)
+
         # plot all the elements onto the respective axes
         reference = [
-            field.plot(kind=kind, ax=ax, action="create", **kwargs, **sp_args)
-            for field, ax, sp_args in zip(self.fields, axs, subplot_args)
+            field.plot(kind=knd, ax=ax, action="create", **kwargs, **sp_args)
+            for field, knd, ax, sp_args in zip(self.fields, kind, axs, subplot_args)
         ]
 
         # return the references for all subplots
