@@ -6,6 +6,7 @@ Defines a scalar field over a grid
 
 import numbers
 from pathlib import Path
+from typing import List  # @UnusedImport
 from typing import TYPE_CHECKING, Callable, Dict, Optional, Sequence, Union
 
 import numpy as np
@@ -274,6 +275,8 @@ class ScalarField(DataFieldBase):
             ScalarField: The projected data in a scalar field with a subgrid of
             the original grid.
         """
+        if isinstance(axes, str):
+            axes = [axes]
         if any(ax not in self.grid.axes for ax in axes):
             raise ValueError(
                 f"The axes {axes} are not all contained in {self.grid} with axes "
@@ -362,7 +365,7 @@ class ScalarField(DataFieldBase):
 
         # obtain the sliced data
         if method == "nearest":
-            idx = []
+            idx: List[Union[int, slice]] = []
             for i in range(grid.num_axes):
                 if i in ax_remove:
                     pos = pos_values[i]
@@ -372,7 +375,7 @@ class ScalarField(DataFieldBase):
                             f"Position {grid.axes[i]} = {pos} is outside the domain"
                         )
                     # add slice that is closest to pos
-                    idx.append(np.argmin((grid.axes_coords[i] - pos) ** 2))
+                    idx.append(int(np.argmin((grid.axes_coords[i] - pos) ** 2)))
                 else:
                     idx.append(slice(None))
             subdata = self.data[tuple(idx)]
