@@ -4,6 +4,7 @@
 
 import numpy as np
 import pytest
+
 from pde import FieldCollection, ScalarField, Tensor2Field, UnitGrid, VectorField
 from pde.fields.base import FieldBase
 from pde.tools.misc import skipUnlessModule
@@ -49,7 +50,9 @@ def test_collections():
     vf.data = 1
     tf.data = 1
     np.testing.assert_allclose(fields.data, 1)
-    assert [np.allclose(i, 1) for i in fields.integrals]
+    assert all(np.allclose(i, 12) for i in fields.integrals)
+    assert all(np.allclose(i, 1) for i in fields.averages)
+    assert np.allclose(fields.magnitudes, np.sqrt([1, 2, 4]))
 
     assert sf.data.shape == (3, 4)
     assert vf.data.shape == (2, 3, 4)
@@ -220,3 +223,20 @@ def test_collection_1_field():
     assert fc.labels == ["a"]
 
     fc.plot()
+
+
+def test_collection_plotting():
+    """ test simple plotting of various fields on various grids """
+    grid = UnitGrid([5])
+    s1 = ScalarField(grid, label="s1")
+    s2 = ScalarField(grid)
+    fc = FieldCollection([s1, s2])
+
+    # test setting different figure sizes
+    fc.plot(figsize="default")
+    fc.plot(figsize="auto")
+    fc.plot(figsize=(3, 3))
+
+    # test different arrangements
+    fc.plot(arrangement="horizontal")
+    fc.plot(arrangement="vertical")
