@@ -164,7 +164,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
                 to the center are returned.
 
         Returns:
-            :class:`numpy.ndarray`: The coordinates of the point
+            :class:`~numpy.ndarray`: The coordinates of the point
         """
         # handle the boundary distance
         r_min = boundary_distance if avoid_center else 0
@@ -187,10 +187,10 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
             return point
 
     def get_line_data(self, data: np.ndarray, extract: str = "auto") -> Dict[str, Any]:
-        """return a line cut along the cylindrical symmetry axis
+        """return a line cut for the cylindrical grid
 
         Args:
-            data (:class:`numpy.ndarray`):
+            data (:class:`~numpy.ndarray`):
                 The values at the grid points
             extract (str):
                 Determines which cut is done through the grid. Possible choices
@@ -242,7 +242,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         """return a 2d-image of the data
 
         Args:
-            data (:class:`numpy.ndarray`): The values at the grid points
+            data (:class:`~numpy.ndarray`): The values at the grid points
 
         Returns:
             A dictionary with information about the image, which is  convenient
@@ -250,7 +250,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         """
         bounds_r, bounds_z = self.axes_bounds
         return {
-            "data": np.vstack((data[::-1, :], data)).T,
+            "data": np.vstack((data[::-1, :], data)),
             "x": self.axes_coords[0],
             "y": self.axes_coords[1],
             "extent": (-bounds_r[1], bounds_r[1], bounds_z[0], bounds_z[1]),
@@ -258,16 +258,19 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
             "label_y": self.axes[1],
         }
 
-    def contains_point(self, point: np.ndarray) -> bool:
+    def contains_point(self, point: np.ndarray) -> np.ndarray:
         """check whether the point is contained in the grid
 
         Args:
-            point (:class:`numpy.ndarray`): Coordinates of the point
+            point (:class:`~numpy.ndarray`): Coordinates of the point
         """
-        assert len(point) == 3
-        r = np.hypot(point[0], point[1])
+        point = np.atleast_1d(point)
+        assert point.shape[-1] == 3
+
+        in_radius = np.hypot(point[..., 0], point[..., 1]) <= self.radius
         bounds_z = self.axes_bounds[1]
-        return bool(r <= self.radius and bounds_z[0] <= point[2] <= bounds_z[1])
+        in_z = (bounds_z[0] <= point[..., 2]) & (point[..., 2] <= bounds_z[1])
+        return in_radius & in_z  # type: ignore
 
     def iter_mirror_points(
         self, point: np.ndarray, with_self: bool = False, only_periodic: bool = True
@@ -275,7 +278,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         """generates all mirror points corresponding to `point`
 
         Args:
-            point (:class:`numpy.ndarray`): the point within the grid
+            point (:class:`~numpy.ndarray`): the point within the grid
             with_self (bool): whether to include the point itself
             only_periodic (bool): whether to only mirror along periodic axes
 
@@ -293,7 +296,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
 
     @cached_property()
     def cell_volume_data(self) -> Tuple[np.ndarray, float]:
-        """ :class:`numpy.ndarray`: the volumes of all cells """
+        """ :class:`~numpy.ndarray`: the volumes of all cells """
         dr, dz = self.discretization
         rs = np.arange(self.shape[0] + 1) * dr
         areas = np.pi * rs ** 2
@@ -304,11 +307,11 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         """convert coordinates of a point to Cartesian coordinates
 
         Args:
-            points (:class:`numpy.ndarray`):
+            points (:class:`~numpy.ndarray`):
                 Points given in the coordinates of the grid
 
         Returns:
-            :class:`numpy.ndarray`: The Cartesian coordinates of the point
+            :class:`~numpy.ndarray`: The Cartesian coordinates of the point
         """
         points = np.atleast_1d(points)
         if points.shape[-1] != self.num_axes:
@@ -326,11 +329,11 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         y-coordinate will be zero.
 
         Args:
-            points (:class:`numpy.ndarray`):
+            points (:class:`~numpy.ndarray`):
                 Points given in Cartesian coordinates.
 
         Returns:
-            :class:`numpy.ndarray`: Points given in the coordinates of the grid
+            :class:`~numpy.ndarray`: Points given in the coordinates of the grid
         """
         points = np.atleast_1d(points)
         assert points.shape[-1] == self.dim
@@ -346,7 +349,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         y-coordinate will be zero.
 
         Args:
-            cells (:class:`numpy.ndarray`):
+            cells (:class:`~numpy.ndarray`):
                 Indices of the cells whose center coordinates are requested.
                 This can be float values to indicate positions relative to the
                 cell center.
@@ -355,7 +358,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
                 coordinates or grid coordinates.
 
         Returns:
-            :class:`numpy.ndarray`: The center points of the respective cells
+            :class:`~numpy.ndarray`: The center points of the respective cells
         """
         cells = np.atleast_1d(cells)
 
@@ -370,7 +373,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         if cartesian:
             return self.point_to_cartesian(points)
         else:
-            return points
+            return points  # type: ignore
 
     def point_to_cell(self, points: np.ndarray) -> np.ndarray:
         """Determine cell(s) corresponding to given point(s)
@@ -380,17 +383,17 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         non-periodic axes).
 
         Args:
-            points (:class:`numpy.ndarray`): Real coordinates
+            points (:class:`~numpy.ndarray`): Real coordinates
 
         Returns:
-            :class:`numpy.ndarray`: The indices of the respective cells
+            :class:`~numpy.ndarray`: The indices of the respective cells
         """
         points = self.point_from_cartesian(points)
 
         # convert from grid coordinates to cells indices
         points[..., 1] -= self.axes_bounds[1][0]
         points /= self.discretization
-        return points.astype(np.int)
+        return points.astype(np.intc)
 
     def difference_vector_real(self, p1: np.ndarray, p2: np.ndarray) -> np.ndarray:
         """return the vector pointing from p1 to p2.
@@ -398,26 +401,24 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
         In case of periodic boundary conditions, the shortest vector is returned
 
         Args:
-            p1 (:class:`numpy.ndarray`): First point(s)
-            p2 (:class:`numpy.ndarray`): Second point(s)
+            p1 (:class:`~numpy.ndarray`): First point(s)
+            p2 (:class:`~numpy.ndarray`): Second point(s)
 
         Returns:
-            :class:`numpy.ndarray`: The difference vectors between the points
+            :class:`~numpy.ndarray`: The difference vectors between the points
             with periodic  boundary conditions applied.
         """
         diff = np.atleast_1d(p2) - np.atleast_1d(p1)
         if self._periodic_z:
             size = self.length
             diff[..., 1] = (diff[..., 1] + size / 2) % size - size / 2
-        return diff
+        return diff  # type: ignore
 
-    def polar_coordinates_real(
-        self, origin: np.ndarray, ret_angle: bool = False
-    ) -> np.ndarray:
+    def polar_coordinates_real(self, origin: np.ndarray, *, ret_angle: bool = False):
         """return spherical coordinates associated with the grid
 
         Args:
-            origin (:class:`numpy.ndarray`): Coordinates of the origin at which the polar
+            origin (:class:`~numpy.ndarray`): Coordinates of the origin at which the polar
                 coordinate system is anchored. Note that this must be of the
                 form `[0, 0, z_val]`, where only `z_val` can be chosen freely.
             ret_angle (bool): Determines whether the azimuthal angle is returned
@@ -433,7 +434,7 @@ class CylindricalGrid(GridBase):  # lgtm [py/missing-equals]
             raise RuntimeError("Origin must lie on symmetry axis for cylindrical grid")
 
         # calculate the difference vector between all cells and the origin
-        diff = self.difference_vector_real([0, origin[2]], self.cell_coords)
+        diff = self.difference_vector_real(np.array([0, origin[2]]), self.cell_coords)
         dist = np.linalg.norm(diff, axis=-1)  # get distance
 
         if ret_angle:
