@@ -19,18 +19,13 @@ from typing import Any, Callable, Tuple
 
 import numba as nb
 import numpy as np
-from scipy import ndimage, sparse
 
+from ... import config
 from ...tools.docstrings import fill_in_docstring
 from ...tools.numba import jit_allocate_out
 from ..boundaries import Boundaries
 from ..cartesian import CartesianGridBase
-from .common import (
-    PARALLELIZATION_THRESHOLD_2D,
-    PARALLELIZATION_THRESHOLD_3D,
-    make_general_poisson_solver,
-    make_laplace_from_matrix,
-)
+from .common import make_general_poisson_solver, make_laplace_from_matrix
 
 
 @fill_in_docstring
@@ -45,6 +40,8 @@ def _get_laplace_matrix_1d(bcs) -> Tuple[Any, Any]:
         tuple: A sparse matrix and a sparse vector that can be used to evaluate
         the discretized laplacian
     """
+    from scipy import sparse
+
     dim_x = bcs.grid.shape[0]
     matrix = sparse.dok_matrix((dim_x, dim_x))
     vector = sparse.dok_matrix((dim_x, 1))
@@ -86,6 +83,8 @@ def _get_laplace_matrix_2d(bcs) -> Tuple[Any, Any]:
         tuple: A sparse matrix and a sparse vector that can be used to evaluate
         the discretized laplacian
     """
+    from scipy import sparse
+
     dim_x, dim_y = bcs.grid.shape
     matrix = sparse.dok_matrix((dim_x * dim_y, dim_x * dim_y))
     vector = sparse.dok_matrix((dim_x * dim_y, 1))
@@ -268,6 +267,8 @@ def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     Returns:
         A function that can be applied to an array of values
     """
+    from scipy import ndimage
+
     scaling = bcs._uniform_discretization ** -2
     args = bcs._scipy_border_mode
 
@@ -343,7 +344,7 @@ def _make_laplace_numba_2d(bcs: Boundaries) -> Callable:
     region_y = bcs[1].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y >= PARALLELIZATION_THRESHOLD_2D ** 2
+    parallel = dim_x * dim_y >= config["numba.parallel_threshold"]
 
     @jit_allocate_out(parallel=parallel)
     def laplace(arr, out=None):
@@ -403,7 +404,7 @@ def _make_laplace_numba_3d(bcs: Boundaries) -> Callable:
     region_z = bcs[2].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y * dim_z >= PARALLELIZATION_THRESHOLD_3D ** 3
+    parallel = dim_x * dim_y * dim_z >= config["numba.parallel_threshold"]
 
     @jit_allocate_out(parallel=parallel)
     def laplace(arr, out=None):
@@ -485,6 +486,8 @@ def _make_gradient_scipy_nd(bcs: Boundaries) -> Callable:
     Returns:
         A function that can be applied to an array of values
     """
+    from scipy import ndimage
+
     scaling = 0.5 / bcs.grid.discretization
     dim = bcs.grid.dim
     shape_out = (dim,) + bcs.grid.shape
@@ -550,7 +553,7 @@ def _make_gradient_numba_2d(bcs: Boundaries) -> Callable:
     region_y = bcs[1].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y >= PARALLELIZATION_THRESHOLD_2D ** 2
+    parallel = dim_x * dim_y >= config["numba.parallel_threshold"]
 
     @jit_allocate_out(parallel=parallel, out_shape=(2, dim_x, dim_y))
     def gradient(arr, out=None):
@@ -587,7 +590,7 @@ def _make_gradient_numba_3d(bcs: Boundaries) -> Callable:
     region_z = bcs[2].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y * dim_z >= PARALLELIZATION_THRESHOLD_3D ** 3
+    parallel = dim_x * dim_y * dim_z >= config["numba.parallel_threshold"]
 
     @jit_allocate_out(parallel=parallel, out_shape=(3, dim_x, dim_y, dim_z))
     def gradient(arr, out=None):
@@ -723,7 +726,7 @@ def _make_gradient_squared_numba_2d(bcs: Boundaries, central: bool = True) -> Ca
     region_y = bcs[1].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y >= PARALLELIZATION_THRESHOLD_2D ** 2
+    parallel = dim_x * dim_y >= config["numba.parallel_threshold"]
 
     if central:
         # use central differences
@@ -787,7 +790,7 @@ def _make_gradient_squared_numba_3d(bcs: Boundaries, central: bool = True) -> Ca
     region_z = bcs[2].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y * dim_z >= PARALLELIZATION_THRESHOLD_3D ** 3
+    parallel = dim_x * dim_y * dim_z >= config["numba.parallel_threshold"]
 
     if central:
         # use central differences
@@ -882,6 +885,8 @@ def _make_divergence_scipy_nd(bcs: Boundaries) -> Callable:
     Returns:
         A function that can be applied to an array of values
     """
+    from scipy import ndimage
+
     shape = bcs.grid.shape
     scaling = 0.5 / bcs.grid.discretization
     args = bcs._scipy_border_mode
@@ -950,7 +955,7 @@ def _make_divergence_numba_2d(bcs: Boundaries) -> Callable:
     region_y = bcs[1].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y >= PARALLELIZATION_THRESHOLD_2D ** 2
+    parallel = dim_x * dim_y >= config["numba.parallel_threshold"]
 
     @jit_allocate_out(parallel=parallel, out_shape=(dim_x, dim_y))
     def divergence(arr, out=None):
@@ -988,7 +993,7 @@ def _make_divergence_numba_3d(bcs: Boundaries) -> Callable:
     region_z = bcs[2].make_region_evaluator()
 
     # use parallel processing for large enough arrays
-    parallel = dim_x * dim_y * dim_z >= PARALLELIZATION_THRESHOLD_3D ** 3
+    parallel = dim_x * dim_y * dim_z >= config["numba.parallel_threshold"]
 
     @jit_allocate_out(parallel=parallel, out_shape=(dim_x, dim_y, dim_z))
     def divergence(arr, out=None):
@@ -1066,6 +1071,8 @@ def _make_vector_gradient_scipy_nd(bcs: Boundaries) -> Callable:
     Returns:
         A function that can be applied to an array of values
     """
+    from scipy import ndimage
+
     scaling = 0.5 / bcs.grid.discretization
     args = bcs._scipy_border_mode
     dim = bcs.grid.dim
@@ -1219,6 +1226,8 @@ def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     Returns:
         A function that can be applied to an array of values
     """
+    from scipy import ndimage
+
     scaling = bcs._uniform_discretization ** -2
     args = bcs._scipy_border_mode
     dim = bcs.grid.dim
@@ -1366,6 +1375,8 @@ def _make_tensor_divergence_scipy_nd(bcs: Boundaries) -> Callable:
     Returns:
         A function that can be applied to an array of values
     """
+    from scipy import ndimage
+
     scaling = 0.5 / bcs.grid.discretization
     args = bcs._scipy_border_mode
     dim = bcs.grid.dim
