@@ -6,26 +6,16 @@ Common functions that are used by many operators
 
 import logging
 import warnings
-from typing import Callable
+from typing import Callable, Optional
 
 import numpy as np
-from scipy import sparse
-
-# Package-wide constant defining when to use parallel numba
-PARALLELIZATION_THRESHOLD_2D = 256
-""" int: threshold for determining when parallel code is created for
-differential operators. The value gives the minimal number of support points in
-each direction for a 2-dimensional grid """
-PARALLELIZATION_THRESHOLD_3D = 64
-""" int: threshold for determining when parallel code is created for
-differential operators. The value gives the minimal number of support points in
-each direction for a 3-dimensional grid """
-
 
 logger = logging.getLogger(__name__)
 
 
-def make_laplace_from_matrix(matrix, vector) -> Callable:
+def make_laplace_from_matrix(
+    matrix, vector
+) -> Callable[[np.ndarray, Optional[np.ndarray]], np.ndarray]:
     """make a Laplace operator using matrix vector products
 
     Args:
@@ -48,15 +38,17 @@ def make_laplace_from_matrix(matrix, vector) -> Callable:
         """ apply the laplace operator to `arr` """
         result = mat.dot(arr.flat) + vec
         if out is None:
-            return result.reshape(arr.shape)
+            return result.reshape(arr.shape)  # type: ignore
         else:
-            out.flat[:] = result
+            out[:] = result.reshape(arr.shape)
             return out
 
     return laplace
 
 
-def make_general_poisson_solver(matrix, vector, method: str = "auto") -> Callable:
+def make_general_poisson_solver(
+    matrix, vector, method: str = "auto"
+) -> Callable[[np.ndarray, Optional[np.ndarray]], np.ndarray]:
     """make an operator that solves Poisson's problem
 
     Args:
@@ -74,6 +66,9 @@ def make_general_poisson_solver(matrix, vector, method: str = "auto") -> Callabl
         solution to Poisson's equation where the array is used as the right hand
         side
     """
+    from scipy import sparse
+    from scipy.sparse.linalg.dsolve.linsolve import MatrixRankWarning
+
     if method not in {"auto", "scipy"}:
         raise ValueError(f"Method {method} is not available")
 
@@ -92,7 +87,7 @@ def make_general_poisson_solver(matrix, vector, method: str = "auto") -> Callabl
                 warnings.simplefilter("error")  # enable warning catching
                 result = sparse.linalg.spsolve(mat, rhs)
 
-        except sparse.linalg.dsolve.linsolve.MatrixRankWarning:
+        except MatrixRankWarning:
             # this can happen for singular laplace matrix, e.g. when pure
             # Neumann conditions are considered. In this case, a solution is
             # obtained using least squares
@@ -121,10 +116,10 @@ def make_general_poisson_solver(matrix, vector, method: str = "auto") -> Callabl
             logger.info("Solved Poisson problem with sparse.linalg.lsmr")
 
         # convert the result to the correct format
-        if out is not None:
-            out.flat[:] = result
-        else:
+        if out is None:
             out = result.reshape(arr.shape)
+        else:
+            out[:] = result.reshape(arr.shape)
         return out
 
     return solve_poisson
