@@ -10,7 +10,6 @@ from abc import ABCMeta
 from typing import TYPE_CHECKING, Any, Dict, Generator, Tuple, Union
 
 import numpy as np
-from scipy import interpolate
 
 from ..tools.cache import cached_property
 from ..tools.docstrings import fill_in_docstring
@@ -128,18 +127,18 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
 
     @cached_property()
     def cell_volume_data(self) -> Tuple[np.ndarray]:
-        """ tuple of :class:`numpy.ndarray`: the volumes of all cells """
+        """ tuple of :class:`~numpy.ndarray`: the volumes of all cells """
         dr = self.discretization[0]
         rs = self.axes_coords[0]
         volumes_h = volume_from_radius(rs + 0.5 * dr, dim=self.dim)
         volumes_l = volume_from_radius(rs - 0.5 * dr, dim=self.dim)
-        return ((volumes_h - volumes_l).reshape(self.shape[0]),)  # type: ignore
+        return ((volumes_h - volumes_l).reshape(self.shape[0]),)
 
     def contains_point(self, point: np.ndarray) -> np.ndarray:
         """check whether the point is contained in the grid
 
         Args:
-            point (:class:`numpy.ndarray`): Coordinates of the point
+            point (:class:`~numpy.ndarray`): Coordinates of the point
         """
         point = np.atleast_1d(point)
         if point.shape[-1] != self.dim:
@@ -147,7 +146,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         r = np.linalg.norm(point, axis=-1)
 
         r_inner, r_outer = self.axes_bounds[0]
-        return r_inner <= r <= r_outer
+        return r_inner <= r <= r_outer  # type: ignore
 
     def get_random_point(
         self,
@@ -170,7 +169,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
                 to the center are returned.
 
         Returns:
-            :class:`numpy.ndarray`: The coordinates of the point
+            :class:`~numpy.ndarray`: The coordinates of the point
         """
         # handle the boundary distance
         r_inner, r_outer = self.axes_bounds[0]
@@ -193,7 +192,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         """return a line cut along the radial axis
 
         Args:
-            data (:class:`numpy.ndarray`):
+            data (:class:`~numpy.ndarray`):
                 The values at the grid points
             extract (str):
                 Determines which cut is done through the grid. This parameter is
@@ -224,7 +223,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         """return a 2d-image of the data
 
         Args:
-            data (:class:`numpy.ndarray`):
+            data (:class:`~numpy.ndarray`):
                 The values at the grid points
             performance_goal (str):
                 Determines the method chosen for interpolation. Possible options
@@ -234,12 +233,14 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
                 or outside the region).
             masked (bool):
                 Whether a :class:`numpy.ma.MaskedArray` is returned for the data
-                instead of the normal :class:`numpy.ndarray`.
+                instead of the normal :class:`~numpy.ndarray`.
 
         Returns:
             A dictionary with information about the image, which is  convenient
             for plotting.
         """
+        from scipy import interpolate
+
         _, r_outer = self.axes_bounds[0]
         r_data = self.axes_coords[0]
 
@@ -292,30 +293,29 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         """generates all mirror points corresponding to `point`
 
         Args:
-            point (:class:`numpy.ndarray`): the point within the grid
+            point (:class:`~numpy.ndarray`): the point within the grid
             with_self (bool): whether to include the point itself
             only_periodic (bool): whether to only mirror along periodic axes
 
         Returns:
             A generator yielding the coordinates that correspond to mirrors
         """
-        point = np.asanyarray(point, dtype=np.double)
         if with_self:
-            yield point
+            yield np.asanyarray(point, dtype=np.double)
 
     def point_from_cartesian(self, points: np.ndarray) -> np.ndarray:
         """convert points given in Cartesian coordinates to this grid
 
         Args:
-            points (:class:`numpy.ndarray`):
+            points (:class:`~numpy.ndarray`):
                 Points given in Cartesian coordinates.
 
         Returns:
-            :class:`numpy.ndarray`: Points given in the coordinates of the grid
+            :class:`~numpy.ndarray`: Points given in the coordinates of the grid
         """
         points = np.atleast_1d(points)
         assert points.shape[-1] == self.dim
-        return np.linalg.norm(points, axis=-1, keepdims=True)
+        return np.linalg.norm(points, axis=-1, keepdims=True)  # type: ignore
 
     def cell_to_point(self, cells: np.ndarray, cartesian: bool = True) -> np.ndarray:
         """convert cell coordinates to real coordinates
@@ -324,7 +324,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         y-coordinate will be zero.
 
         Args:
-            cells (:class:`numpy.ndarray`):
+            cells (:class:`~numpy.ndarray`):
                 Indices of the cells whose center coordinates are requested.
                 This can be float values to indicate positions relative to the
                 cell center.
@@ -333,7 +333,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
                 coordinates or grid coordinates.
 
         Returns:
-            :class:`numpy.ndarray`: The center points of the respective cells
+            :class:`~numpy.ndarray`: The center points of the respective cells
         """
         cells = np.atleast_1d(cells)
         assert cells.shape[-1] == self.num_axes
@@ -344,22 +344,22 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         if cartesian:
             return self.point_to_cartesian(points)
         else:
-            return points
+            return points  # type: ignore
 
     def point_to_cell(self, points: np.ndarray) -> np.ndarray:
         """Determine cell(s) corresponding to given point(s)
 
         Args:
-            points (:class:`numpy.ndarray`): Real coordinates
+            points (:class:`~numpy.ndarray`): Real coordinates
 
         Returns:
-            :class:`numpy.ndarray`: The indices of the respective cells
+            :class:`~numpy.ndarray`: The indices of the respective cells
         """
         # convert from grid coordinates to cells indices
         r = self.point_from_cartesian(points)
         r_inner, _ = self.axes_bounds[0]
         cells = (r - r_inner) / self.discretization[0]
-        return cells.astype(np.int)
+        return cells.astype(np.intc)  # type: ignore
 
     def difference_vector_real(self, p1: np.ndarray, p2: np.ndarray) -> np.ndarray:
         """return the vector pointing from p1 to p2.
@@ -367,36 +367,37 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         In case of periodic boundary conditions, the shortest vector is returned
 
         Args:
-            p1 (:class:`numpy.ndarray`): First point(s)
-            p2 (:class:`numpy.ndarray`): Second point(s)
+            p1 (:class:`~numpy.ndarray`): First point(s)
+            p2 (:class:`~numpy.ndarray`): Second point(s)
 
         Returns:
-            :class:`numpy.ndarray`: The difference vectors between the points
+            :class:`~numpy.ndarray`: The difference vectors between the points
                 with periodic boundary conditions applied.
         """
-        return np.atleast_1d(p2) - np.atleast_1d(p1)
+        return np.atleast_1d(p2) - np.atleast_1d(p1)  # type: ignore
 
-    def polar_coordinates_real(self, origin=[0, 0, 0], ret_angle: bool = False):
+    def polar_coordinates_real(self, origin=None, *, ret_angle: bool = False, **kwargs):
         """return spherical coordinates associated with the grid
 
         Args:
-            origin (:class:`numpy.ndarray`): Coordinates of the origin at which the polar
-                coordinate system is anchored. Note that this must be of the
-                form `[0, 0, z_val]`, where only `z_val` can be chosen freely.
-            ret_angle (bool): Determines whether angles are returned alongside
-                the distance. If `False` only the distance to the origin is
-                returned for each support point of the grid.
-                If `True`, the distance and angles are returned. Note that in
-                the case of spherical grids, this angle is zero by convention.
+            origin:
+                Place holder variable to comply with the interface
+            ret_angle (bool):
+                Determines whether angles are returned alongside the distance. If
+                `False` only the distance to the origin is returned for each support
+                point of the grid. If `True`, the distance and angles are returned. Note
+                that in the case of spherical grids, this angle is zero by convention.
         """
-        origin = np.array(origin, dtype=np.double, ndmin=1)
-        if not np.array_equal(origin, np.zeros(self.dim)):
-            raise RuntimeError(f"Origin must be {str([0]*self.dim)}")
+        # check the consistency of the origin argument, which can be set for other grids
+        if origin is not None:
+            origin = np.array(origin, dtype=np.double, ndmin=1)
+            if not np.array_equal(origin, np.zeros(self.dim)):
+                raise RuntimeError(f"Origin must be {str([0]*self.dim)}")
 
         # the distance to the origin is exactly the radial coordinate
         rs = self.axes_coords[0]
         if ret_angle:
-            return rs, np.zeros_like(rs)
+            return rs, (np.zeros_like(rs),) * (self.dim - 1)
         else:
             return rs
 
@@ -456,26 +457,29 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
             return bcs
 
     def get_cartesian_grid(self, mode: str = "valid", num: int = None) -> CartesianGrid:
-        """return a Cartesian grid for this Cylindrical one
+        """return a Cartesian grid for this spherical one
 
         Args:
             mode (str):
-                Determines how the grid is determined. Setting it to 'valid'
-                only returns points that are fully resolved in the spherical
-                grid, e.g., the sphere is circumscribed. Conversely, 'full'
-                returns all data, so the sphere is inscribed.
+                Determines how the grid is determined. Setting it to 'valid' (or
+                'inscribed') only returns points that are fully resolved in the
+                spherical grid, e.g., the Cartesian grid is inscribed in the sphere.
+                Conversely, 'full' (or 'circumscribed') returns all data, so the
+                Cartesian grid is circumscribed.
             num (int):
                 Number of support points along each axis of the returned grid.
 
         Returns:
             :class:`pde.grids.cartesian.CartesianGrid`: The requested grid
         """
-        # Pick the grid instance
+        # pick how the grid is determined
         if mode == "valid":
             if self.has_hole:
                 self._logger.warn("The sphere has holes, so not all points are valid")
             bounds = self.radius / np.sqrt(self.dim)
-        elif mode == "full":
+        elif mode == "inscribed":
+            bounds = self.radius / np.sqrt(self.dim)
+        elif mode == "full" or mode == "circumscribed":
             bounds = self.radius
         else:
             raise ValueError(f"Unsupported mode `{mode}`")
@@ -486,6 +490,38 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):  # lgtm [py/missing-equals
         grid_bounds = [(-bounds, bounds)] * self.dim
         return CartesianGrid(grid_bounds, num)
 
+    @plot_on_axes()
+    def plot(self, ax, **kwargs):
+        r"""visualize the spherically symmetric grid in two dimensions
+
+        Args:
+            {PLOT_ARGS}
+            \**kwargs: Extra arguments are passed on the to the matplotlib
+                plotting routines, e.g., to set the color of the lines
+        """
+        from matplotlib import collections, patches
+
+        kwargs.setdefault("edgecolor", kwargs.get("color", "k"))
+        kwargs.setdefault("facecolor", "none")
+        (rb,) = self.axes_bounds
+        rmax = rb[1]
+
+        # draw circular parts
+        circles = []
+        for r in np.linspace(*rb, self.shape[0] + 1):
+            if r == 0:
+                c = patches.Circle((0, 0), 0.01 * rmax)
+            else:
+                c = patches.Circle((0, 0), r)
+            circles.append(c)
+        ax.add_collection(collections.PatchCollection(circles, **kwargs))
+
+        ax.set_xlim(-rmax, rmax)
+        ax.set_xlabel("x")
+        ax.set_ylim(-rmax, rmax)
+        ax.set_ylabel("y")
+        ax.set_aspect(1)
+
 
 class PolarGrid(SphericalGridBase):
     r"""2-dimensional polar grid assuming angular symmetry
@@ -517,7 +553,7 @@ class PolarGrid(SphericalGridBase):
         coordinates will be zero.
 
         Returns:
-            :class:`numpy.ndarray`: The Cartesian coordinates of the point
+            :class:`~numpy.ndarray`: The Cartesian coordinates of the point
         """
         points = np.atleast_1d(points)
         if points.shape[-1] != self.num_axes:
@@ -527,33 +563,6 @@ class PolarGrid(SphericalGridBase):
         x = np.zeros_like(y)
         return np.stack((x, y), axis=-1)
 
-    @plot_on_axes()
-    def plot(self, ax, **kwargs):
-        r"""visualize the polar grid
-
-        Args:
-            {PLOT_ARGS}
-            \**kwargs: Extra arguments are passed on the to the matplotlib
-                plotting routines, e.g., to set the color of the lines
-        """
-        from matplotlib import patches
-
-        kwargs.setdefault("color", "k")
-        (rb,) = self.axes_bounds
-        rmax = rb[1]
-
-        for r in np.linspace(*rb, self.shape[0] + 1):
-            if r == 0:
-                ax.plot(0, 0, ".", **kwargs)
-            else:
-                c = patches.Circle((0, 0), r, fill=False, **kwargs)
-                ax.add_artist(c)
-        ax.set_xlim(-rmax, rmax)
-        ax.set_xlabel("x")
-        ax.set_ylim(-rmax, rmax)
-        ax.set_ylabel("y")
-        ax.set_aspect(1)
-
 
 class SphericalGrid(SphericalGridBase):
     r"""3-dimensional spherical grid assuming spherical symmetry
@@ -585,11 +594,11 @@ class SphericalGrid(SphericalGridBase):
         coordinates will be zero.
 
         Args:
-            points (:class:`numpy.ndarray`):
+            points (:class:`~numpy.ndarray`):
                 Points given in the coordinates of the grid
 
         Returns:
-            :class:`numpy.ndarray`: The Cartesian coordinates of the point
+            :class:`~numpy.ndarray`: The Cartesian coordinates of the point
         """
         points = np.atleast_1d(points)
         if points.shape[-1] != self.num_axes:
