@@ -11,11 +11,12 @@ from typing import Any, Callable, Dict
 import numpy as np
 
 from ..fields import FieldCollection, VectorField
-from ..fields.base import DataFieldBase, FieldBase, OptionalArrayLike
+from ..fields.base import DataFieldBase, FieldBase
 from ..grids.boundaries.axes import BoundariesData
 from ..pdes.base import PDEBase
 from ..tools.docstrings import fill_in_docstring
 from ..tools.numba import jit, nb
+from ..tools.typing import ArrayLike
 
 
 class PDE(PDEBase):
@@ -23,25 +24,25 @@ class PDE(PDEBase):
 
     Attributes:
         variables (tuple):
-            The name of the variables (i.e., fields) in the order they are
-            expected to appear in the `state`.
+            The name of the variables (i.e., fields) in the order they are expected to
+            appear in the `state`.
         diagnostics (dict):
-            Additional diagnostic information that might help with analyzing
-            problems, e.g., when :mod:`sympy` cannot parse or :mod`numba` cannot
-            compile a function.
+            Additional diagnostic information that might help with analyzing problems,
+            e.g., when :mod:`sympy` cannot parse or :mod`numba` cannot compile a
+            function.
     """
 
     @fill_in_docstring
     def __init__(
         self,
         rhs: "OrderedDict[str, str]",
-        noise: OptionalArrayLike = 0,
+        noise: ArrayLike = 0,
         bc: BoundariesData = "natural",
         bc_ops: "OrderedDict[str, BoundariesData]" = None,
         user_funcs: Dict[str, Any] = None,
         consts: Dict[str, Any] = None,
     ):
-        """
+        r"""
         Warning:
             {WARNING_EXEC}
 
@@ -59,7 +60,7 @@ class PDE(PDEBase):
                 field. Moreover, the dot product between two vector fields can be
                 denoted by using `dot(field1, field2)` in the expression, while an outer
                 product is calculated using `outer(field1, field2)`.
-            noise (float or :class:`numpy.ndarray`):
+            noise (float or :class:`~numpy.ndarray`):
                 Magnitude of additive Gaussian white noise. The default value of zero
                 implies deterministic partial differential equations will be solved.
                 Different noise magnitudes can be supplied for each field in coupled
@@ -76,8 +77,9 @@ class PDE(PDEBase):
                 The keys follow the format "VARIABLE:OPERATOR", where VARIABLE specifies
                 the expression in `rhs` where the boundary condition is applied to the
                 operator specified by OPERATOR. For both identifiers, the wildcard
-                symbol "*" denotes that all fields and operators are affected,
-                respectively.
+                symbol "\*" denotes that all fields and operators are affected,
+                respectively. For instance, the identifier "c:\*" allows specifying a
+                condition for all operators of the field named `c`.
             user_funcs (dict, optional):
                 A dictionary with user defined functions that can be used in the
                 expressions in `rhs`.
@@ -191,8 +193,8 @@ class PDE(PDEBase):
     ) -> Dict[str, Any]:
         """prepare the expression by setting internal variables in the cache
 
-        Note that the expensive calculations in this method are only carried
-        out if the state attributes change.
+        Note that the expensive calculations in this method are only carried out if the
+        state attributes change.
 
         Args:
             state (:class:`~pde.fields.FieldBase`):
@@ -397,8 +399,8 @@ class PDE(PDEBase):
 
         Returns:
             A function with signature `(state_data, t)`, which can be called
-            with an instance of :class:`numpy.ndarray` of the state data and
-            the time to obtained an instance of :class:`numpy.ndarray` giving
+            with an instance of :class:`~numpy.ndarray` of the state data and
+            the time to obtained an instance of :class:`~numpy.ndarray` giving
             the evolution rate.
         """
         num_fields = len(state)
@@ -462,10 +464,9 @@ class PDE(PDEBase):
                 An example for the state defining the grid and data types
 
         Returns:
-            A function with signature `(state_data, t)`, which can be called
-            with an instance of :class:`numpy.ndarray` of the state data and
-            the time to obtained an instance of :class:`numpy.ndarray` giving
-            the evolution rate.
+            A function with signature `(state_data, t)`, which can be called with an
+            instance of :class:`~numpy.ndarray` of the state data and the time to
+            obtained an instance of :class:`~numpy.ndarray` giving the evolution rate.
         """
         cache = self._prepare_cache(state, backend="numba")
 
