@@ -25,7 +25,7 @@ from ..fields import FieldCollection, ScalarField, Tensor2Field, VectorField
 from ..fields.base import FieldBase
 from ..grids.base import GridBase
 from ..tools.docstrings import fill_in_docstring
-from ..tools.misc import display_progress
+from ..tools.output import display_progress
 from ..trackers.base import InfoDict, TrackerBase
 from ..trackers.intervals import IntervalData, IntervalType
 
@@ -42,8 +42,9 @@ class StorageBase(metaclass=ABCMeta):
     fields in order and indivdual time points can also be accessed.
     """
 
-    times: Sequence[float]  # :class:`numpy.ndarray`): stored time points
-    data: Any  # The actual data for all the stored times
+    times: Sequence[float]  # :class:`~numpy.ndarray`): stored time points
+    data: Any  # actual data for all the stored times
+    write_mode: str  # mode determining how the storage behaves
 
     def __init__(self, info: InfoDict = None, write_mode: str = "truncate_once"):
         """
@@ -51,13 +52,12 @@ class StorageBase(metaclass=ABCMeta):
             info (dict):
                 Supplies extra information that is stored in the storage
             write_mode (str):
-                Determines how new data is added to already existing one.
-                Possible values are: 'append' (data is always appended),
-                'truncate' (data is cleared every time this storage is used
-                for writing), or 'truncate_once' (data is cleared for the first
-                writing, but subsequent data using the same instances are
-                appended). Alternatively, specifying 'readonly' will disable
-                writing completely.
+                Determines how new data is added to already existing one. Possible
+                values are: 'append' (data is always appended), 'truncate' (data is
+                cleared every time this storage is used for writing), or 'truncate_once'
+                (data is cleared for the first writing, but subsequent data using the
+                same instances are appended). Alternatively, specifying 'readonly' will
+                disable writing completely.
         """
         self.info = {} if info is None else info
         self.write_mode = write_mode
@@ -281,29 +281,34 @@ class StorageBase(metaclass=ABCMeta):
         """ finalize the storage after writing """
         pass
 
-    def extract_field(self, field_index: int) -> "MemoryStorage":
-        """extract the time course of a single field in a collection
+    def extract_field(self, field_index: int, label: str = None) -> "MemoryStorage":
+        """extract the time course of a single field from a collection
 
         Note:
-            This might return a view into the original data, so modifying the
-            returned data can also change the underlying original data.
+            This might return a view into the original data, so modifying the returned
+            data can also change the underlying original data.
 
         Args:
             field_index (index):
-                The index into the field collection
+                The index into the field collection. This determines which field of the
+                collection is returned.
+            label (str):
+                The label of the returned field. If omitted, the stored label is used.
 
         Returns:
-            :class:`MemoryStorage`: a storage instance that contains the data
-            for the single field
+            :class:`MemoryStorage`: a storage instance that contains the data for the
+            single field
         """
         from .memory import MemoryStorage  # @Reimport
 
         # get the field to check its type
         if not isinstance(self._field, FieldCollection):
-            raise TypeError("Can only extract fields from FieldCollections")
+            raise TypeError("Can only extract fields from `FieldCollection`")
 
         # extract the field and the associated time series
-        field_obj = self._field[field_index]
+        field_obj = self._field[field_index].copy()
+        if label:
+            field_obj.label = label
         field_slice = self._field._slices[field_index]
         data = [d[field_slice].reshape(field_obj.data.shape) for d in self.data]
 
@@ -318,18 +323,16 @@ class StorageBase(metaclass=ABCMeta):
         """extract a particular time interval
 
         Note:
-            This might return a view into the original data, so modifying the
-            returned data can also change the underlying original data.
+            This might return a view into the original data, so modifying the returned
+            data can also change the underlying original data.
 
         Args:
             t_range (float or tuple):
-                Determines the range of time points included in the result. If
-                only a single number is given, all data up to this time point
-                are included.
+                Determines the range of time points included in the result. If only a
+                single number is given, all data up to this time point are included.
 
         Returns:
-            :class:`MemoryStorage`: a storage instance that contains the
-            extracted data.
+            :class:`MemoryStorage`: a storage instance that contains the extracted data.
         """
         from .memory import MemoryStorage  # @Reimport
 
@@ -349,8 +352,8 @@ class StorageBase(metaclass=ABCMeta):
 
         # extract the actual memory
         return MemoryStorage(
-            times=self.times[i_start:i_end],
-            data=self.data[i_start:i_end],
+            times=self.times[i_start:i_end],  # type: ignore
+            data=self.data[i_start:i_end],  # type: ignore
             field_obj=self._field,
             info=self.info,
         )
