@@ -1,5 +1,5 @@
 """
-Module containing functions for managing cache structures
+Functions, classes, and decorators for managing caches
 
 .. autosummary::
    :nosignatures:
@@ -14,7 +14,7 @@ Module containing functions for managing cache structures
    SerializedDict
 
 
-.. codeauthor:: David Zwicker <dzwicker@seas.harvard.edu>
+.. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 """
 
 from __future__ import division
@@ -44,9 +44,9 @@ def objects_equal(a, b) -> bool:
     """
     # compare numpy arrays
     if isinstance(a, np.ndarray):
-        return np.array_equal(a, b)  # type: ignore
+        return np.array_equal(a, b)
     if isinstance(b, np.ndarray):
-        return np.array_equal(b, a)  # type: ignore
+        return np.array_equal(b, a)
 
     # compare dictionaries
     if isinstance(a, dict):
