@@ -4,8 +4,8 @@ Miscallenous python functions
 .. autosummary::
    :nosignatures:
 
-   module_available
    environment
+   module_available
    ensure_directory_exists
    preserve_scalars
    decorator_arguments
@@ -15,6 +15,9 @@ Miscallenous python functions
    hybridmethod
    estimate_computation_speed
    hdf_write_attributes
+   number
+   get_common_dtype
+   number_array
 
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 """
@@ -24,18 +27,36 @@ import functools
 import importlib
 import json
 import os
-import sys
 import unittest
+import warnings
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Sequence, Union
+from typing import Any, Callable, Dict, Sequence, Union
 
 import numpy as np
 
-# import functions moved on 2020-07-27
-# using this path for import is deprecated
-from .output import display_progress, get_progress_bar_class  # @UnusedImport
+from .typing import ArrayLike, Number
 
-Number = Union[float, complex]
+
+def environment(dict_type=dict) -> Dict[str, Any]:
+    """obtain information about the compute environment
+
+    Args:
+        dict_type: The type to create the returned dictionaries. The default is
+            `dict`, but :class:`collections.OrderedDict` is an alternative.
+
+    Returns:
+        dict: information about the python installation and packages
+    """
+    # import functions moved on 2020-02-08
+    # using this path for import is deprecated
+    from .config import environment  # @UnusedImport
+
+    warnings.warn(
+        "Importing `environment` from `pde.tools.misc` is deprecated. Import it from "
+        "`pde` directly instead.",
+        DeprecationWarning,
+    )
+    return environment(dict_type)
 
 
 def module_available(module_name: str) -> bool:
@@ -55,58 +76,6 @@ def module_available(module_name: str) -> bool:
         return True
 
 
-def environment(dict_type=dict) -> Dict[str, Any]:
-    """obtain information about the compute environment
-
-    Args:
-        dict_type: The type to create the returned dictionaries. The default is
-            `dict`, but :class:`collections.OrderedDict` is an alternative.
-
-    Returns:
-        dict: information about the python installation and packages
-    """
-    import matplotlib as mpl
-
-    from .. import __version__ as package_version
-    from .numba import numba_environment
-    from .plotting import get_plotting_context
-
-    def get_package_versions(packages: List[str]) -> Dict[str, str]:
-        """ tries to load certain python packages and returns their version """
-        versions: Dict[str, str] = dict_type()
-        for name in sorted(packages):
-            try:
-                module = importlib.import_module(name)
-            except ImportError:
-                versions[name] = "not available"
-            else:
-                versions[name] = module.__version__  # type: ignore
-        return versions
-
-    result: Dict[str, Any] = dict_type()
-    result["package version"] = package_version
-    result["python version"] = sys.version
-    result["platform"] = sys.platform
-
-    # add details for mandatory packages
-    result["mandatory packages"] = get_package_versions(
-        ["matplotlib", "numba", "numpy", "scipy", "sympy"]
-    )
-    result["matplotlib environment"] = {
-        "backend": mpl.get_backend(),
-        "plotting context": get_plotting_context().__class__.__name__,
-    }
-
-    # add details about optional packages
-    result["optional packages"] = get_package_versions(
-        ["h5py", "pandas", "pyfftw", "tqdm"]
-    )
-    if module_available("numba"):
-        result["numba environment"] = numba_environment()
-
-    return result
-
-
 def ensure_directory_exists(folder: Union[str, Path]):
     """creates a folder if it not already exists
 
@@ -380,7 +349,7 @@ def number(value: Union[Number, str]) -> Number:
     Result:
         Number: A complex number or a float if the imaginary part vanishes
     """
-    result = complex(value)
+    result = complex(value)  # type: ignore
     return result.real if result.imag == 0 else result
 
 
@@ -390,19 +359,19 @@ def get_common_dtype(*args):
     Args:
         *args: All items (arrays, scalars, etc) to be checked
 
-    Returns: np.complex if any entry is complex, otherwise np.double
+    Returns: numpy.cdouble if any entry is complex, otherwise np.double
     """
     for arg in args:
         if np.iscomplexobj(arg):
-            return np.complex
+            return np.cdouble
     return np.double
 
 
-def number_array(data: np.ndarray, dtype=None, copy: bool = True) -> np.ndarray:
-    """convert array dtype either to np.double or np.complex
+def number_array(data: ArrayLike, dtype=None, copy: bool = True) -> np.ndarray:
+    """convert an array with arbitrary dtype either to np.double or np.cdouble
 
     Args:
-        data (:class:`numpy.ndarray`):
+        data (:class:`~numpy.ndarray`):
             The data that needs to be converted to a float array. This can also be any
             iterable of numbers.
         dtype (numpy dtype):
@@ -413,7 +382,7 @@ def number_array(data: np.ndarray, dtype=None, copy: bool = True) -> np.ndarray:
             untouched). Note that data will always be copied when changing the dtype.
 
     Returns:
-        :class:`numpy.ndarray`: An array with the correct dtype
+        :class:`~numpy.ndarray`: An array with the correct dtype
     """
     if dtype is None:
         # dtype needs to be determined automatically
@@ -424,7 +393,7 @@ def number_array(data: np.ndarray, dtype=None, copy: bool = True) -> np.ndarray:
             # Conversion can fail when `data` contains a complex sympy number, i.e.,
             # sympy.I. In this case, we simply try to convert the expression using a
             # complex dtype
-            result = np.array(data, dtype=np.complex, copy=copy)
+            result = np.array(data, dtype=np.cdouble, copy=copy)
 
     else:
         # a specific dtype is requested
